@@ -275,22 +275,6 @@ static void process_intput_buffer(pitch_t pitch)
         pitch->acc[i] += pitch->fade[pitch->step-i-1]*pitch->tmp[i];
 }
 
-static st_sample_t clip(pitch_t pitch, PITCH_FLOAT v)
-{
-    if (v < ST_SAMPLE_MIN)
-    {
-        pitch->clipped++;
-        return ST_SAMPLE_MIN;
-    }
-    else if (v > ST_SAMPLE_MAX)
-    {
-        pitch->clipped++;
-        return ST_SAMPLE_MAX;
-    }
-    else
-        return (st_sample_t) v;
-}
-
 /*
  * Process options
  */
@@ -403,6 +387,8 @@ int st_pitch_start(eff_t effp)
 
     /* computer inner stuff... */
 
+    pitch->state = pi_input;
+
     /* Should I trust pow?
      * BTW, the twelve's root of two is 1.0594630943592952645618252,
      * if we consider an equal temperament.
@@ -542,7 +528,13 @@ int st_pitch_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
             int toout = MIN(*osamp-oindex, pitch->step-pitch->iacc);
 
             for (i=0; i<toout; i++)
-                obuf[oindex++] = clip(pitch, pitch->acc[pitch->iacc++]);
+            {
+                float f;
+
+                f = pitch->acc[pitch->iacc++];
+                ST_SAMPLE_CLIP_COUNT(f, pitch->clipped);
+                obuf[oindex++] = f;
+            }
 
             if (pitch->iacc == pitch->step)
             {
@@ -589,12 +581,21 @@ int st_pitch_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
 
     /* (pitch->state == pi_output) */
     for (i=0; i<*osamp && i<pitch->index-pitch->overlap;)
-        obuf[i++] = clip(pitch, pitch->acc[pitch->iacc++]);
+    {
+        float f;
+
+        f = pitch->acc[pitch->iacc++];
+        ST_SAMPLE_CLIP_COUNT(f, pitch->clipped);
+        obuf[i++] = f;
+    }
 
     /* report... */
     *osamp = i;
 
-    return ST_SUCCESS;
+    if ((pitch->index - pitch->overlap) == 0)
+        return ST_EOF;
+    else
+        return ST_SUCCESS;
 }
     
 /*
