@@ -357,13 +357,8 @@ int st_resample_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
                 // orig: *obuf++ = r->Y[i] * ISCALE;
                 Float ftemp = r->Y[i] * ISCALE;
 
-                if (ftemp > ST_SAMPLE_MAX)
-                        *obuf = ST_SAMPLE_MAX;
-                else if (ftemp < ST_SAMPLE_MIN)
-                        *obuf = ST_SAMPLE_MIN;
-                else
-                        *obuf = ftemp;
-                obuf++;
+                ST_SAMPLE_CLIP(ftemp);
+                *obuf++ = ftemp;
         }
 
         *isamp = Nx;
@@ -406,6 +401,9 @@ int st_resample_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
         /* fprintf(stderr,"DRAIN osamp %d\n", *osamp); */
         if (isamp_res)
                 st_warn("drain overran obuf by %d\n", isamp_res); fflush(stderr);
+        /* FIXME: This is very picky.  IF obuf is not big enough to
+         * drain remaining samples, they will be lost.
+         */
         return (ST_EOF);
 }
 
