@@ -25,6 +25,7 @@
 #define NAMELEN    30           /* Size of Samplevision name */
 #define COMMENTLEN 60           /* Size of Samplevision comment, not shared */
 #define MIDI_UNITY 60           /* MIDI note number to play sample at unity */
+#define MARKERLEN  10           /* Size of Marker name */
 
 /* The header preceeding the sample data */
 struct smpheader {
@@ -45,8 +46,8 @@ struct loop {
 
 /* Samplevision marker definition structure */
 struct marker {
-        char name[10];          /* Ascii Marker name */
-        uint32_t position;      /* Sample Number, not byte number */
+        char name[MARKERLEN + 1]; /* Ascii Marker name */
+        uint32_t position;        /* Sample Number, not byte number */
 };
 
 /* The trailer following the sample data */
@@ -92,11 +93,12 @@ static int readtrailer(ft_t ft, struct smptrailer *trailer)
                 ft->loops[i].count = trailer->loops[i].count;
         }
         for(i = 0; i < 8; i++) {        /* read the 8 markers */
-                if (st_read(ft, trailer->markers[i].name, 1, 10) != 10)
+                if (st_readbuf(ft, trailer->markers[i].name, 1, MARKERLEN) != 10)
                 {
                     st_fail_errno(ft,ST_EHDR,"EOF in SMP");
                     return(ST_EOF);
                 }
+                trailer->markers[i].name[MARKERLEN] = 0;
                 st_readdw(ft, &(trailer->markers[i].position));
         }
         st_readb(ft, (unsigned char *)&(trailer->MIDInote));
@@ -186,10 +188,10 @@ int st_smpseek(ft_t ft, st_size_t offset)
         new_offset += (channel_block - alignment);
     new_offset += smp->dataStart;
 
-    ft->st_errno = st_seek(ft, new_offset, SEEK_SET);
+    ft->st_errno = st_seeki(ft, new_offset, SEEK_SET);
 
     if( ft->st_errno == ST_SUCCESS )
-        smp->NoOfSamps = ft->length - new_offset;
+        smp->NoOfSamps = ft->length - (new_offset / ft->info.size);
 
     return(ft->st_errno);
 }
@@ -224,7 +226,7 @@ int st_smpstartread(ft_t ft)
         }
 
         /* Read SampleVision header */
-        if (st_read(ft, (char *)&header, 1, HEADERSIZE) != HEADERSIZE)
+        if (st_readbuf(ft, (char *)&header, 1, HEADERSIZE) != HEADERSIZE)
         {
                 st_fail_errno(ft,ST_EHDR,"unexpected EOF in SMP header");
                 return(ST_EOF);
@@ -260,8 +262,8 @@ int st_smpstartread(ft_t ft)
         samplestart = st_tell(ft);
 
         /* seek from the current position (the start of sample data) by */
-        /* NoOfSamps * 2 */
-        if (st_seek(ft, smp->NoOfSamps * 2L, 1) == -1)
+        /* NoOfSamps * sizeof(int16_t) */
+        if (st_seeki(ft, smp->NoOfSamps * 2L, 1) == -1)
         {
                 st_fail_errno(ft,errno,"SMP unable to seek to trailer");
                 return(ST_EOF);
@@ -273,7 +275,7 @@ int st_smpstartread(ft_t ft)
         }
 
         /* seek back to the beginning of the data */
-        if (st_seek(ft, samplestart, 0) == -1) 
+        if (st_seeki(ft, samplestart, 0) == -1) 
         {
                 st_fail_errno(ft,errno,"SMP unable to seek back to start of sample data");
                 return(ST_EOF);
@@ -384,7 +386,7 @@ int st_smpstartwrite(ft_t ft)
         sprintf(header.name, "%-*.*s", NAMELEN, NAMELEN, ft->comment);
 
         /* Write file header */
-        if(st_write(ft, &header, 1, HEADERSIZE) != HEADERSIZE)
+        if(st_writebuf(ft, &header, 1, HEADERSIZE) != HEADERSIZE)
         {
             st_fail_errno(ft,errno,"SMP: Can't write header completely");
             return(ST_EOF);
@@ -419,7 +421,7 @@ int st_smpstopwrite(ft_t ft)
         /* Assign the trailer data */
         settrailer(ft, &trailer, ft->info.rate);
         writetrailer(ft, &trailer);
-        if (st_seek(ft, 112, 0) == -1)
+        if (st_seeki(ft, 112, 0) == -1)
         {
                 st_fail_errno(ft,errno,"SMP unable to seek back to save size");
                 return(ST_EOF);
