@@ -157,6 +157,7 @@ static struct st_effect efftab[MAX_EFF]; /* left/mono channel effects */
 static struct st_effect efftabR[MAX_EFF];/* right channel effects */
 static int neffects;                     /* # of effects to run on data */
 static int input_eff;                    /* last input effect with data */
+static int input_eff_eof;                /* has input_eff reached EOF? */
 
 static struct st_effect user_efftab[MAX_USER_EFF];
 static int nuser_effects;
@@ -595,6 +596,7 @@ static void process(void) {
         file_desc[f]->st_errno = 0;
 
     input_eff = 0;
+    input_eff_eof = 0;
 
     /* mark chain as empty */
     for(e = 1; e < neffects; e++)
@@ -703,16 +705,7 @@ static void process(void) {
                     {
                         double sample;
                         sample = efftab[0].obuf[s] + ibuf[f][s];
-                        if (sample < ST_SAMPLE_MIN)
-                        {
-                            sample = ST_SAMPLE_MIN;
-                            clipped++;
-                        }
-                        else if (sample > ST_SAMPLE_MAX)
-                        {
-                            sample = ST_SAMPLE_MAX;
-                            clipped++;
-                        }
+                        ST_SAMPLE_CLIP_COUNT(sample, clipped);
                         efftab[0].obuf[s] = sample;
                     }
             }
@@ -774,7 +767,7 @@ static void process(void) {
          * we are exiting anyways.
          */
         if (st_close(file_desc[f]) == ST_EOF)
-            st_warn(file_desc[f]->st_errstr);
+            st_warn("%s", file_desc[f]->st_errstr);
     }
 
     if (writing)
@@ -783,7 +776,7 @@ static void process(void) {
          * are exiting anyways.
          */
         if (st_close(file_desc[file_count-1]) == ST_EOF)
-            st_warn(file_desc[file_count-1]->st_errstr);
+            st_warn("%s", file_desc[file_count-1]->st_errstr);
     }
 }
 
@@ -1060,6 +1053,13 @@ static int flow_effect_out(void)
       /* this is because buffering system isn't a nice queueing system */
       for(e = neffects - 1; e >= input_eff; e--)
       {
+          /* Do not call flow effect on input if its reported
+           * EOF already as thats a waste of time and may
+           * do bad things.
+           */
+          if (e == input_eff && input_eff_eof)
+              continue;
+
           /* flow_effect returns ST_EOF when it will not process
            * any more samples.  This is used to bail out early.
            * Since we are "pulling" data, it is OK that we are not
@@ -1068,7 +1068,11 @@ static int flow_effect_out(void)
            */
           flowstatus  = flow_effect(e);
           if (flowstatus == ST_EOF)
+          {
               input_eff = e+1;
+              /* Assume next effect hasn't reach EOF yet */
+              input_eff_eof = 0;
+          }
 
           /* If this buffer contains more input data then break out
            * of this loop now.  This will allow us to loop back around
@@ -1107,6 +1111,12 @@ static int flow_effect_out(void)
           total = 0;
           do
           {
+              /* Do not do any more writing during user aborts as
+               * we may be stuck in an infinite writing loop.
+               */
+              if (user_abort)
+                  return ST_EOF;
+
               len = st_write(file_desc[file_count-1], 
                              &efftab[neffects-1].obuf[total],
                              (st_ssize_t)efftab[neffects-1].olen-total);
@@ -1164,6 +1174,15 @@ static int flow_effect_out(void)
 
       if (!havedata && input_eff > 0)
       {
+          /* When EOF has been detected, skip to the next input
+           * before looking for more data.
+           */
+          if (input_eff_eof)
+          {
+              input_eff++;
+              input_eff_eof = 0;
+          }
+
           /* If the input file is not returning data then
            * we must prime the pump using the drain effect.
            * After its primed, the loop will suck the data
@@ -1176,11 +1195,16 @@ static int flow_effect_out(void)
 
               rc = drain_effect(input_eff);
 
-              if (rc == ST_EOF || efftab[input_eff].olen == 0)
+              if (efftab[input_eff].olen == 0)
+              {
                   input_eff++;
+                  /* Assume next effect hasn't reached EOF yet. */
+                  input_eff_eof = 0;
+              }
               else
               {
                   havedata = 1;
+                  input_eff_eof = (rc == ST_EOF) ? 1 : 0;
                   break;
               }
           }
@@ -1208,6 +1232,12 @@ static int flow_effect(int e)
     st_sample_t *ibuf, *obuf;
     int effstatus;
 
+    /* Do not attempt to do any more effect processing during
+     * user aborts as we may be stuck in an infinit flow loop.
+     */
+    if (user_abort)
+        return ST_EOF;
+
     /* I have no input data ? */
     if (efftab[e-1].odone == efftab[e-1].olen)
     {
@@ -1311,7 +1341,11 @@ static int drain_effect_out(void)
 {
     /* Skip past input effect since we know thats not needed */
     if (input_eff == 0)
+    {
         input_eff = 1;
+        /* Assuming next effect hasn't reached EOF yet. */
+        input_eff_eof = 0;
+    }
 
     /* Try to prime the pump with some data */
     while (input_eff < neffects)
@@ -1320,10 +1354,17 @@ static int drain_effect_out(void)
 
         rc = drain_effect(input_eff);
 
-        if (rc == ST_EOF || efftab[input_eff].olen == 0)
+        if (efftab[input_eff].olen == 0)
+        {
             input_eff++;
+            /* Assuming next effect hasn't reached EOF yet. */
+            input_eff_eof = 0;
+        }
         else
+        {
+            input_eff_eof = (rc == ST_EOF) ? 1 : 0;
             break;
+        }
     }
 
     /* Just do standard flow routines after the priming. */
@@ -1496,14 +1537,7 @@ static st_sample_t volumechange(st_sample_t *buf, st_ssize_t ct,
         top = buf+ct;
         while (p < top) {
             y = vol * *p;
-            if (y < ST_SAMPLE_MIN) {
-                y = ST_SAMPLE_MIN;
-                clips++;
-            }
-            else if (y > ST_SAMPLE_MAX) {
-                y = ST_SAMPLE_MAX;
-                clips++;
-            }
+            ST_SAMPLE_CLIP_COUNT(y, clips);
             *p++ = y;
         }
         return clips;
