@@ -127,8 +127,15 @@ int st_vorbisstartread(ft_t ft)
         /* Record audio info */
         ft->info.rate = vi->rate;
         ft->info.size = ST_SIZE_16BIT;
-        ft->info.encoding = ST_ENCODING_SIGN2;
+        ft->info.encoding = ST_ENCODING_VORBIS;
         ft->info.channels = vi->channels;
+
+        /* ov_pcm_total doesn't work on non-seekable files so
+         * skip that step in that case.  Also, it reports
+         * "frame"-ish results so we must * channels.
+         */
+        if (ft->seekable)
+            ft->length = ov_pcm_total(vb->vf, -1) * ft->info.channels;
         
         /* Record comments */
         if (vc->comments == 0)
@@ -273,8 +280,8 @@ int st_vorbisstopread(ft_t ft)
 int oe_write_page(ogg_page *page, ft_t ft)
 {
         int written;
-        written = st_write(ft, page->header,1,page->header_len);
-        written += st_write(ft, page->body,1,page->body_len);
+        written = st_writebuf(ft, page->header,1,page->header_len);
+        written += st_writebuf(ft, page->body,1,page->body_len);
 
         return written;
 }
@@ -344,6 +351,9 @@ int st_vorbisstartwrite(ft_t ft)
         vorbis_enc_t *ve;
         long rate;
 
+        ft->info.size = ST_SIZE_16BIT;
+        ft->info.encoding = ST_ENCODING_VORBIS;
+
         /* Allocate memory for all of the structures */
         ve = vb->vorbis_enc_data = (vorbis_enc_t *)malloc(sizeof(vorbis_enc_t));
         if (ve == NULL)
@@ -429,7 +439,7 @@ st_ssize_t st_vorbiswrite(ft_t ft, st_sample_t *buf, st_ssize_t len)
                 }
         }
 
-        return (ST_SUCCESS);    
+        return (len);    
 }
 
 int st_vorbisstopwrite(ft_t ft) 
