@@ -87,8 +87,8 @@ typedef struct wavstuff {
     unsigned short samplesPerBlock;
     unsigned short blockAlign;
     st_size_t dataStart;  /* need to for seeking */
-    int found_cooledit;
-    
+    int found_cooledit;   
+
     /* following used by *ADPCM wav files */
     unsigned short nCoefs;          /* ADPCM: number of coef sets */
     short         *iCoefs;          /* ADPCM: coef sets           */
@@ -135,7 +135,7 @@ unsigned short  ImaAdpcmReadBlock(ft_t ft)
     int samplesThisBlock;
 
     /* Pull in the packet and check the header */
-    bytesRead = st_read(ft, wav->packet, 1, wav->blockAlign);
+    bytesRead = st_readbuf(ft, wav->packet, 1, wav->blockAlign);
     samplesThisBlock = wav->samplesPerBlock;
     if (bytesRead < wav->blockAlign) 
     { 
@@ -176,7 +176,7 @@ unsigned short  AdpcmReadBlock(ft_t ft)
     const char *errmsg;
 
     /* Pull in the packet and check the header */
-    bytesRead = st_read(ft, wav->packet, 1, wav->blockAlign);
+    bytesRead = st_readbuf(ft, wav->packet, 1, wav->blockAlign);
     samplesThisBlock = wav->samplesPerBlock;
     if (bytesRead < wav->blockAlign) 
     {
@@ -222,7 +222,7 @@ static int xxxAdpcmWriteBlock(ft_t ft)
             ImaBlockMashI(chans, wav->samples, wav->samplesPerBlock, wav->state, wav->packet, 9);
         }
         /* write the compressed packet */
-        if (st_write(ft, wav->packet, wav->blockAlign, 1) != 1)
+        if (st_writebuf(ft, wav->packet, wav->blockAlign, 1) != 1)
         {
             st_fail_errno(ft,ST_EOF,"write error");
             return (ST_EOF);
@@ -294,7 +294,7 @@ st_ssize_t wavgsmread(ft_t ft, st_sample_t *buf, st_ssize_t len)
   /* read and decode loop, possibly leaving some samples in wav->gsmsample */
     while (done < len) {
         wav->gsmindex=0;
-        bytes = st_read(ft, frame, 1, 65);   
+        bytes = st_readbuf(ft, frame, 1, 65);   
         if (bytes <=0)
             return done;
         if (bytes<65) {
@@ -335,7 +335,7 @@ static int wavgsmflush(ft_t ft)
     gsm_encode(wav->gsmhandle, wav->gsmsample, frame);
     /*encode the odd half long (33 byte) frame */
     gsm_encode(wav->gsmhandle, wav->gsmsample+160, frame+32);
-    if (st_write(ft, frame, 1, 65) != 65)
+    if (st_writebuf(ft, frame, 1, 65) != 65)
     {
         st_fail_errno(ft,ST_EOF,"write error");
         return (ST_EOF);
@@ -394,11 +394,9 @@ void wavgsmstopwrite(ft_t ft)
 /****************************************************************************/
 /* General Sox WAV file code                                                */
 /****************************************************************************/
-static st_ssize_t findChunk(ft_t ft, const char *Label)
+static int findChunk(ft_t ft, const char *Label, st_size_t *len)
 {
     char magic[5];
-    st_ssize_t len;
-    uint32_t tmp_len;
     for (;;)
     {
         if (st_reads(ft, magic, 4) == ST_EOF)
@@ -407,21 +405,26 @@ static st_ssize_t findChunk(ft_t ft, const char *Label)
                           Label);
             return ST_EOF;
         }
-        st_readdw(ft, &tmp_len);
-        len = tmp_len;
         st_report("WAV Chunk %s", magic);
+        if (st_readdw(ft, len) == ST_EOF)
+        {
+            st_fail_errno(ft, ST_EHDR, "WAVE file %s chunk is to short", 
+                          magic);
+            return ST_EOF;
+        }
 
         if (strncmp(Label, magic, 4) == 0)
             break; /* Found the data chunk */
 
         /* skip to next chunk */
-        if (st_seek(ft, len, SEEK_CUR) != ST_SUCCESS)
+        if (st_seeki(ft, *len, SEEK_CUR) != ST_SUCCESS)
         {
-            st_fail_errno(ft,ST_EHDR, "WAV chunk appears to have invalid size %d.", len);
+            st_fail_errno(ft,ST_EHDR, 
+                          "WAV chunk appears to have invalid size %d.", *len);
             return ST_EOF;
         }
     }
-    return len;
+    return ST_SUCCESS;
 }
 
 /*
@@ -457,12 +460,20 @@ int st_wavstartread(ft_t ft)
 
     if (ST_IS_BIGENDIAN) ft->swap = ft->swap ? 0 : 1;
 
-    if (st_reads(ft, magic, 4) == ST_EOF || strncmp("RIFF", magic, 4))
+    if (st_reads(ft, magic, 4) == ST_EOF || (strncmp("RIFF", magic, 4) != 0 &&
+                                             strncmp("RIFX", magic, 4) != 0))
     {
         st_fail_errno(ft,ST_EHDR,"WAVE: RIFF header not found");
         return ST_EOF;
     }
 
+    /* RIFX is a Big-endian RIFF */
+    if (strncmp("RIFX", magic, 4) == 0) 
+    {
+        st_report("Found RIFX header, swapping bytes");
+        ft->swap = ft->swap ? 0 : 1;
+    }
+
     st_readdw(ft, &dwRiffLength);
 
     if (st_reads(ft, magic, 4) == ST_EOF || strncmp("WAVE", magic, 4))
@@ -472,8 +483,12 @@ int st_wavstartread(ft_t ft)
     }
 
     /* Now look for the format chunk */
-    wFmtSize = len = findChunk(ft, "fmt ");
-    /* findChunk() only returns if chunk was found */
+    if (findChunk(ft, "fmt ", &len) == ST_EOF)
+    {
+        st_fail_errno(ft,ST_EHDR,"WAVE chunk fmt not found");
+        return ST_EOF;
+    }
+    wFmtSize = len;
     
     if (wFmtSize < 16)
     {
@@ -628,8 +643,12 @@ int st_wavstartread(ft_t ft)
     wav->packet = NULL;
     wav->samples = NULL;
 
-    /* non-PCM formats have extended fmt chunk.  Check for those cases. */
-    if (wav->formatTag != WAVE_FORMAT_PCM) {
+    /* non-PCM formats expect alaw and mulaw formats have extended fmt chunk.  
+     * Check for those cases. 
+     */
+    if (wav->formatTag != WAVE_FORMAT_PCM &&
+        wav->formatTag != WAVE_FORMAT_ALAW &&
+        wav->formatTag != WAVE_FORMAT_MULAW) {
         if (len >= 2) {
             st_readw(ft, &wExtSize);
             len -= 2;
@@ -834,18 +853,18 @@ int st_wavstartread(ft_t ft)
     }
 
     /* Skip anything left over from fmt chunk */
-    st_seek(ft, len, SEEK_CUR);
+    st_seeki(ft, len, SEEK_CUR);
 
     /* for non-PCM formats, there's a 'fact' chunk before
      * the upcoming 'data' chunk */
 
     /* Now look for the wave data chunk */
-    dwDataLength = len = findChunk(ft, "data");
-    if (len == ST_EOF)
+    if (findChunk(ft, "data", &len) == ST_EOF)
     {
         st_fail_errno(ft, ST_EOF, "Could not find data chunk.");
         return ST_EOF;
     }
+    dwDataLength = len;
 
     /* Data starts here */
     wav->dataStart = st_tell(ft);
@@ -876,16 +895,15 @@ int st_wavstartread(ft_t ft)
 
 #ifdef ENABLE_GSM
     case WAVE_FORMAT_GSM610:
-        wav->numSamples = (((dwDataLength / wav->blockAlign) * wav->samplesPerBlock) * ft->info.channels);
+        wav->numSamples = ((dwDataLength / wav->blockAlign) * wav->samplesPerBlock);
         wavgsminit(ft);
-        ft->length = wav->numSamples;
+        ft->length = wav->numSamples*ft->info.channels;
         break;
 #endif
 
     default:
-        wav->numSamples = dwDataLength/ft->info.size;   /* total samples */
-        ft->length = wav->numSamples;
-
+        wav->numSamples = dwDataLength/ft->info.size/ft->info.channels;
+        ft->length = wav->numSamples*ft->info.channels;
     }
 
     st_report("Reading Wave file: %s format, %d channel%s, %d samp/sec",
@@ -897,25 +915,27 @@ int st_wavstartread(ft_t ft)
     /* Can also report extended fmt information */
     switch (wav->formatTag)
     {
-    case WAVE_FORMAT_ADPCM:
-        st_report("        %d Extsize, %d Samps/block, %d bytes/block %d Num Coefs",
-                wExtSize,wav->samplesPerBlock,bytesPerBlock,wav->nCoefs);
-        break;
+        case WAVE_FORMAT_ADPCM:
+            st_report("        %d Extsize, %d Samps/block, %d bytes/block %d Num Coefs, %d Samps/chan",
+                      wExtSize,wav->samplesPerBlock,bytesPerBlock,wav->nCoefs,
+                      wav->numSamples);
+            break;
 
-    case WAVE_FORMAT_IMA_ADPCM:
-        st_report("        %d Extsize, %d Samps/block, %d bytes/block",
-                wExtSize,wav->samplesPerBlock,bytesPerBlock);
-        break;
+        case WAVE_FORMAT_IMA_ADPCM:
+            st_report("        %d Extsize, %d Samps/block, %d bytes/block %d Samps/chan",
+                      wExtSize, wav->samplesPerBlock, bytesPerBlock, 
+                      wav->numSamples);
+            break;
 
 #ifdef ENABLE_GSM
-    case WAVE_FORMAT_GSM610:
-        st_report("GSM .wav: %d Extsize, %d Samps/block,  %d samples",
-                wExtSize,wav->samplesPerBlock,wav->numSamples);
-        break;
+        case WAVE_FORMAT_GSM610:
+            st_report("GSM .wav: %d Extsize, %d Samps/block, %d Samples/chan",
+                      wExtSize, wav->samplesPerBlock, wav->numSamples);
+            break;
 #endif
 
-    default:
-        break;
+        default:
+            st_report("        %d Samps/chans", wav->numSamples);
     }
 
     /* Horrible way to find Cool Edit marker points. Taken from Quake source*/
@@ -926,8 +946,8 @@ int st_wavstartread(ft_t ft)
          * doubt any machine writing Cool Edit Chunks writes them at an odd 
          * offset */
         len = (len + 1) & ~1;
-        if (st_seek(ft, len, SEEK_CUR) == ST_SUCCESS &&
-            findChunk(ft, "LIST") != ST_EOF)
+        if (st_seeki(ft, len, SEEK_CUR) == ST_SUCCESS &&
+            findChunk(ft, "LIST", &len) != ST_EOF)
         {
             wav->found_cooledit = 1;
             ft->comment = (char*)malloc(256);
@@ -974,7 +994,7 @@ int st_wavstartread(ft_t ft)
                             strcat(ft->comment,text);
                         }
                         if (strlen(text) < len)
-                           st_seek(ft, len - strlen(text), SEEK_CUR); 
+                           st_seeki(ft, len - strlen(text), SEEK_CUR); 
                     } 
                     else if (strncmp(magic,"ISFT",4) == 0)
                     {
@@ -993,12 +1013,12 @@ int st_wavstartread(ft_t ft)
                             strcat(ft->comment,text);
                         }
                         if (strlen(text) < len)
-                           st_seek(ft, len - strlen(text), SEEK_CUR); 
+                           st_seeki(ft, len - strlen(text), SEEK_CUR); 
                     } 
                     else if (strncmp(magic,"cue ",4) == 0)
                     {
                         st_report("Chunk cue ");
-                        st_seek(ft,len-4,SEEK_CUR);
+                        st_seeki(ft,len-4,SEEK_CUR);
                         st_readdw(ft,&dwLoopPos);
                         ft->loops[0].start = dwLoopPos;
                     } 
@@ -1008,18 +1028,19 @@ int st_wavstartread(ft_t ft)
                         st_readdw(ft,&dwLoopPos);
                         ft->loops[0].length = dwLoopPos - ft->loops[0].start;
                         if (len > 4)
-                           st_seek(ft, len - 4, SEEK_CUR); 
+                           st_seeki(ft, len - 4, SEEK_CUR); 
                     } 
                     else 
                     {
                         st_report("Attempting to seek beyond unsupported chunk '%c%c%c%c' of length %d bytes\n", magic[0], magic[1], magic[2], magic[3], len);
-                        st_seek(ft, len, SEEK_CUR);
+                        len = (len + 1) & ~1;
+                        st_seeki(ft, len, SEEK_CUR);
                     }
                 }
             }
         }
         st_clearerr(ft);
-        st_seek(ft,wav->dataStart,SEEK_SET);
+        st_seeki(ft,wav->dataStart,SEEK_SET);
     }   
     return ST_SUCCESS;
 }
@@ -1046,10 +1067,6 @@ st_ssize_t st_wavread(ft_t ft, st_sample_t *buf, st_ssize_t len)
         case ST_ENCODING_IMA_ADPCM:
         case ST_ENCODING_ADPCM:
 
-            /* FIXME: numSamples is not used consistently in
-             * wav handler.  Sometimes it accounts for stereo,
-             * sometimes it does not.
-             */
             /* See reason for cooledit check in comments below */
             if (wav->found_cooledit && len > (wav->numSamples*ft->info.channels)) 
                 len = (wav->numSamples*ft->info.channels);
@@ -1101,8 +1118,8 @@ st_ssize_t st_wavread(ft_t ft, st_sample_t *buf, st_ssize_t len)
 #ifdef ENABLE_GSM
         case ST_ENCODING_GSM:
             /* See reason for cooledit check in comments below */
-            if (wav->found_cooledit && len > wav->numSamples) 
-                len = wav->numSamples;
+            if (wav->found_cooledit && len > wav->numSamples*ft->info.channels) 
+                len = (wav->numSamples*ft->info.channels);
 
             done = wavgsmread(ft, buf, len);
             if (done == 0 && wav->numSamples != 0)
@@ -1120,8 +1137,8 @@ st_ssize_t st_wavread(ft_t ft, st_sample_t *buf, st_ssize_t len)
              * greater then 2Gig and can't be represented
              * by the 32-bit size field.
              */
-            if (wav->found_cooledit && len > wav->numSamples) 
-                len = wav->numSamples;
+            if (wav->found_cooledit && len > wav->numSamples*ft->info.channels) 
+                len = (wav->numSamples*ft->info.channels);
 
             done = st_rawread(ft, buf, len);
             /* If software thinks there are more samples but I/O */
@@ -1130,7 +1147,14 @@ st_ssize_t st_wavread(ft_t ft, st_sample_t *buf, st_ssize_t len)
                 st_warn("Premature EOF on .wav input file");
         }
 
-        wav->numSamples -= done;
+        /* Only return buffers that contain a totally playable
+         * amount of audio.
+         */
+        done -= (done % (ft->info.channels * ft->info.size));
+        if (done/ft->info.channels > wav->numSamples)
+            wav->numSamples = 0;
+        else
+            wav->numSamples -= (done/ft->info.channels);
         return done;
 }
 
@@ -1174,39 +1198,40 @@ int st_wavstopread(ft_t ft)
 
 int st_wavstartwrite(ft_t ft) 
 {
-        wav_t   wav = (wav_t) ft->priv;
-        int     rc;
+    wav_t wav = (wav_t) ft->priv;
+    int rc;
 
-        ft->st_errno = ST_SUCCESS;
+    ft->st_errno = ST_SUCCESS;
 
-        if (ST_IS_BIGENDIAN) ft->swap = ft->swap ? 0 : 1;
+    if (ST_IS_BIGENDIAN) ft->swap = ft->swap ? 0 : 1;
 
-        /* FIXME: This reserves memory but things could fail
-         * later on and not release this memory.
-         */
-        if (ft->info.encoding != ST_ENCODING_ADPCM &&
-            ft->info.encoding != ST_ENCODING_IMA_ADPCM &&
-            ft->info.encoding != ST_ENCODING_GSM)
-        {
-                rc = st_rawstartwrite(ft);
-                if (rc)
-                    return rc;
-        }
-
-        wav->numSamples = 0;
-        wav->dataLength = 0;
-        if (!ft->seekable)
-                st_warn("Length in output .wav header will be wrong since can't seek to fix it");
-        rc = wavwritehdr(ft, 0);  /* also calculates various wav->* info */
-        if (rc != 0)
+    /* FIXME: This reserves memory but things could fail
+     * later on and not release this memory.
+     */
+    if (ft->info.encoding != ST_ENCODING_ADPCM &&
+        ft->info.encoding != ST_ENCODING_IMA_ADPCM &&
+        ft->info.encoding != ST_ENCODING_GSM)
+    {
+        rc = st_rawstartwrite(ft);
+        if (rc)
             return rc;
+    }
 
-        wav->packet = NULL;
-        wav->samples = NULL;
-        wav->iCoefs = NULL;
-        switch (wav->formatTag)
-        {
+    wav->numSamples = 0;
+    wav->dataLength = 0;
+    if (!ft->seekable)
+        st_warn("Length in output .wav header will be wrong since can't seek to fix it");
+    rc = wavwritehdr(ft, 0);  /* also calculates various wav->* info */
+    if (rc != 0)
+        return rc;
+
+    wav->packet = NULL;
+    wav->samples = NULL;
+    wav->iCoefs = NULL;
+    switch (wav->formatTag)
+    {
         int ch, sbsize;
+
         case WAVE_FORMAT_IMA_ADPCM:
             initImaTable();
         /* intentional case fallthru! */
@@ -1233,14 +1258,14 @@ int st_wavstartwrite(ft_t ft)
 #endif
         default:
             break;
-        }
-        return ST_SUCCESS;
+    }
+    return ST_SUCCESS;
 }
 
 /* wavwritehdr:  write .wav headers as follows:
  
 bytes      variable      description
-0  - 3     'RIFF'
+0  - 3     'RIFF'/'RIFX' Little/Big-endian
 4  - 7     wRiffLength   length of file minus the 8 byte riff header
 8  - 11    'WAVE'
 12 - 15    'fmt '
@@ -1338,7 +1363,7 @@ static int wavwritehdr(ft_t ft, int second_header)
          ft->info.encoding == ST_ENCODING_GSM) &&
          ft->info.size != ST_SIZE_BYTE)
     {
-        st_warn("Overriding output size to bytes for compressed data.");
+        st_report("Overriding output size to bytes for compressed data.");
         ft->info.size = ST_SIZE_BYTE;
     }
 
@@ -1353,7 +1378,7 @@ static int wavwritehdr(ft_t ft, int second_header)
                     ft->info.encoding != ST_ENCODING_ADPCM &&
                     ft->info.encoding != ST_ENCODING_IMA_ADPCM)
             {
-                st_warn("Do not support %s with 8-bit data.  Forcing to unsigned",st_encodings_str[(unsigned char)ft->info.encoding]);
+                st_report("Do not support %s with 8-bit data.  Forcing to unsigned",st_encodings_str[(unsigned char)ft->info.encoding]);
                 ft->info.encoding = ST_ENCODING_UNSIGNED;
             }
             break;
@@ -1361,7 +1386,7 @@ static int wavwritehdr(ft_t ft, int second_header)
             wBitsPerSample = 16;
             if (ft->info.encoding != ST_ENCODING_SIGN2)
             {
-                st_warn("Do not support %s with 16-bit data.  Forcing to Signed.",st_encodings_str[(unsigned char)ft->info.encoding]);
+                st_report("Do not support %s with 16-bit data.  Forcing to Signed.",st_encodings_str[(unsigned char)ft->info.encoding]);
                 ft->info.encoding = ST_ENCODING_SIGN2;
             }
             break;
@@ -1370,13 +1395,13 @@ static int wavwritehdr(ft_t ft, int second_header)
             if (ft->info.encoding != ST_ENCODING_SIGN2 &&
                 ft->info.encoding != ST_ENCODING_FLOAT)
             {
-                st_warn("Do not support %s with 32-bit data.  Forcing to Signed.",st_encodings_str[(unsigned char)ft->info.encoding]);
+                st_report("Do not support %s with 32-bit data.  Forcing to Signed.",st_encodings_str[(unsigned char)ft->info.encoding]);
                 ft->info.encoding = ST_ENCODING_SIGN2;
             }
 
             break;
         default:
-            st_warn("Do not support %s in WAV files.  Forcing to Signed Words.",st_sizes_str[(unsigned char)ft->info.size]);
+            st_report("Do not support %s in WAV files.  Forcing to Signed Words.",st_sizes_str[(unsigned char)ft->info.size]);
             ft->info.encoding = ST_ENCODING_SIGN2;
             ft->info.size = ST_SIZE_WORD;
             wBitsPerSample = 16;
@@ -1434,7 +1459,7 @@ static int wavwritehdr(ft_t ft, int second_header)
 #ifdef ENABLE_GSM
             if (wChannels!=1)
             {
-                st_warn("Overriding GSM audio from %d channel to 1\n",wChannels);
+                st_report("Overriding GSM audio from %d channel to 1\n",wChannels);
                 wChannels = ft->info.channels = 1;
             }
             wFormatTag = WAVE_FORMAT_GSM610;
@@ -1470,7 +1495,6 @@ static int wavwritehdr(ft_t ft, int second_header)
                 /* intentional case fallthrough! */
 #endif
             default:
-                dwSamplesWritten /= wChannels; /* because how rawwrite()'s work */
                 blocksWritten = (dwSamplesWritten+wSamplesPerBlock-1)/wSamplesPerBlock;
                 dwDataLength = blocksWritten * wBlockAlign;
         }
@@ -1492,7 +1516,20 @@ static int wavwritehdr(ft_t ft, int second_header)
     dwAvgBytesPerSec = (double)wBlockAlign*ft->info.rate / (double)wSamplesPerBlock + 0.5;
 
     /* figured out header info, so write it */
-    st_writes(ft, "RIFF");
+
+
+    /* If user specified opposite swap then we think, assume they are
+     * asking to write a RIFX file.
+     */
+    if ((!ST_IS_BIGENDIAN && ft->swap) || 
+        (ST_IS_BIGENDIAN && !ft->swap))
+    {
+        if (!second_header)
+            st_report("Requested to swap bytes so writing  RIFX header");
+        st_writes(ft, "RIFX");
+    }
+    else
+        st_writes(ft, "RIFF");
     st_writedw(ft, wRiffLength);
     st_writes(ft, "WAVE");
     st_writes(ft, "fmt ");
@@ -1595,13 +1632,13 @@ st_ssize_t st_wavwrite(ft_t ft, st_sample_t *buf, st_ssize_t len)
 #ifdef ENABLE_GSM
         case WAVE_FORMAT_GSM610:
             len = wavgsmwrite(ft, buf, len);
-            wav->numSamples += len;
+            wav->numSamples += (len/ft->info.channels);
             return len;
             break;
 #endif
         default:
             len = st_rawwrite(ft, buf, len);
-            wav->numSamples += len; /* must later be divided by wChannels */
+            wav->numSamples += (len/ft->info.channels);
             return len;
         }
 }
@@ -1644,7 +1681,7 @@ int st_wavstopwrite(ft_t ft)
         if (!ft->seekable)
                 return ST_EOF;
 
-        if (st_seek(ft, 0L, SEEK_SET) != 0)
+        if (st_seeki(ft, 0L, SEEK_SET) != 0)
         {
                 st_fail_errno(ft,ST_EOF,"Can't rewind output file to rewrite .wav header.");
                 return ST_EOF;
@@ -1732,10 +1769,11 @@ int st_wavseek(ft_t ft, st_size_t offset)
                 new_offset += (channel_block - alignment);
             new_offset += wav->dataStart;
 
-            ft->st_errno = st_seek(ft, new_offset, SEEK_SET);
+            ft->st_errno = st_seeki(ft, new_offset, SEEK_SET);
 
             if( ft->st_errno == ST_SUCCESS )
-                wav->numSamples = ft->length - new_offset;
+                wav->numSamples = (ft->length / ft->info.channels) -
+                                  (new_offset / ft->info.size / ft->info.channels);
     }
 
     return(ft->st_errno);
